/*
 * Project name:
     GNSS Click board example
 * Copyright:
     (c) mikroElektronika, 2015.
 * Revision History:
     20150417:
       - initial release (DM)
 * Description:
     This example demonstrates the functionality of the GNSS Click board.
     It displays a world map on the Glcd and shows the location of the GNSS module on it.
 * Test configuration:
     MCU:             ATMEGA32
                      http://www.atmel.com/Images/doc2503.pdf
     dev.board:       EasyAVR v7
                      http://www.mikroe.com/easyavr/
     Oscillator:      8.0000 MHz Crystal
     ext. modules:    GNSS click board
                      http://www.mikroe.com/click/gnss/
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/mikroc/avr/
 * NOTES:
     - Place GNSS Click board in the mikroBUS socket 1 on the EasyAVR7 board.
     - Turn on GLCD backlight switch SW.3.1 (board specific)
 */

#include "World Map.h"

sbit RST at PORTA6_bit;
sbit RST_Direction at DDA6_bit;

// Variables used in the example
char txt[768];
char *string;
int i, latitude, longitude;
unsigned short ready;

// Glcd module connections
char GLCD_DataPort at PORTC;
char GLCD_DataPort_Direction at DDRC;

sbit GLCD_CS1 at PORTB0_bit;
sbit GLCD_CS2 at PORTB1_bit;
sbit GLCD_RS  at PORTA2_bit;
sbit GLCD_RW  at PORTA3_bit;
sbit GLCD_EN  at PORTD6_bit;
sbit GLCD_RST at PORTD7_bit;

sbit GLCD_CS1_Direction at DDB0_bit;
sbit GLCD_CS2_Direction at DDB1_bit;
sbit GLCD_RS_Direction  at DDA2_bit;
sbit GLCD_RW_Direction  at DDA3_bit;
sbit GLCD_EN_Direction  at DDD6_bit;
sbit GLCD_RST_Direction at DDD7_bit;
// End Glcd module connections

void interrupt() iv IVT_ADDR_USART__RXC ics ICS_AUTO {
  if (RXC_bit == 1) {              // As long as RXC flag is set
    txt[i] = UART1_Read();         // Read data and store it to txrt string
    i++;                           // Increment string index
    if (i == 768) {                // If index = 768,
      i = 0;                       //   set it to zero
    ready = 1;                     // Ready for parsing GPS data
  }
  RXC_bit = 0;                     // Set RXC to 0
  }
}

void Display_Cursor(signed int lat, signed int lon) {
  unsigned int latitude_y, longitude_x;

  // Latitude and Longitude scaling for 128x64 display:
  // Latitude: Input range is -90 to 90 degrees
  // Longitude: Input range is -180 to 180 degrees
  latitude_y = ((61*(90 - lat))/180) + 1;
  longitude_x = (lon +180) * 125;
  longitude_x = longitude_x/360 + 1;

  // Cursor drawing:
  Glcd_Dot(longitude_x,latitude_y,2);       // Center dot
  Glcd_Dot(longitude_x-1,latitude_y,2);     // Left dot
  Glcd_Dot(longitude_x+1,latitude_y,2);     // Right dot
  Glcd_Dot(longitude_x,latitude_y-1,2);     // Upper dot
  Glcd_Dot(longitude_x,latitude_y+1,2);     // Lower dot
}

void main() {

  RST_Direction = 1;               // Set RST pin as output

  RST = 0;
  Delay_ms(100);                   // Pulldown RST pin at least 10ms
  RST = 1;

  Glcd_Init();                     // Initialize GLCD
  Glcd_Fill(0x00);                 // Clear display
  Delay_ms(100);

  ready = 0;                       // Initialize variables
  i = 0;

  UART1_Init(9600);                // Initialize UART module at 9600

  RXCIE_bit = 1;                   // Enable interrupt on UART receive
  SREG_I_bit = 1;                  // Enable Global interrupt

  Glcd_Image( world_bmp );         // Display World map on the GLCD
   
  while(1) {

    if(ready == 1) {               // If the data in txt array is ready do:
      ready = 0;
      string = strstr(txt,"$GPGGA");
      if(string != 0) {            // If txt array contains "$GPGLL" string we proceed...
        if(string[43] == '1') {     // If "$GPGLL" NMEA message have ',' sign in the 8-th
                                   // position it means that tha GPS receiver does not have FIXed position!


            latitude = (string[18]-48)*10 + (string[19]-48);
            longitude = (string[30]-48)*100 + (string[31]-48)*10 + (string[32]-48);
            if(string[28] == 'S') {           // if the latitude is in the South direction it has minus sign
              latitude = 0 - latitude;
            }
            if(string[41] == 'W') {           // if the longitude is in the West direction it has minus sign
              longitude = 0 - longitude;
            }

          Display_Cursor(latitude, longitude);   // Display the cursor on the world map
        }
      }
    }
  }
}